using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;

namespace WritingGPSApplications.Part3.Example
{
    /// <summary>
    /// Represents the Equidistant Cylindrical map projection.
    /// </summary>
    /// <remarks>Map projections are used to flatten 3D coordinates into a 2D form which can be displayed
    /// easily on a computer screen.  This class provides two methods, Project and Deproject, which convert
    /// convert geographic coordinates to projected coordinates, and vice versa.</remarks>
    public class MapProjection
    {
        /* Many map projections work with coordinates in the form of radians instead of degrees.
         * These constants will help us convert back and forth.
         */
        private const double DEGREEStoRADIANS = Math.PI / 180;
        private const double RADIANStoDEGREES = 180 / Math.PI;
        
        /* These values represent the equatorial radius of the WGS84 ellipsoid in meters.
         * resulting in projected coordinates which are also in meters
         */
        private const double WGS84SEMIMAJOR = 6378137.0;
        private const double ONEOVERWGS84SEMIMAJOR = 1.0 / WGS84SEMIMAJOR;

        public MapProjection()
        { }

        /// <summary>
        /// Converts a geographic coordinate into a projected coordinate.
        /// </summary>
        /// <param name="geographicCoordinate">A <strong>PointF</strong> object, representing a geographic coordinate.</param>
        /// <returns>A projected coordinate in the form of a <strong>PointF</strong> object.</returns>
        public PointF Project(PointF geographicCoordinate)
        {
            // Make a new Point object
            PointF result = new PointF();

            // Calculate the projected X coordinate
            result.X = (float)(geographicCoordinate.X * DEGREEStoRADIANS * Math.Cos(0) * WGS84SEMIMAJOR);

            // Calculate the projected Y coordinate
            result.Y = (float)(geographicCoordinate.Y * DEGREEStoRADIANS * WGS84SEMIMAJOR);

            // Return the result
            return result;
        }

        /// <summary>
        /// Converts a projected coordinate to a geographic coordinate.
        /// </summary>
        /// <param name="projectedCoordinate">A <strong>PointF</strong> object, representing a projected coordinate.</param>
        /// <returns>A geographic coordinate in the form of a <strong>PointF</strong> object.</returns>
        public PointF Deproject(PointF projectedCoordinate)
        {
            // Make a new point to store the result
            PointF result = new PointF();

            // Calculate the geographic X coordinate (longitude)
            result.X = (float)(projectedCoordinate.X * ONEOVERWGS84SEMIMAJOR / Math.Cos(0) * RADIANStoDEGREES);

            // Calculate the geographic Y coordinate (latitude)
            result.Y = (float)(projectedCoordinate.Y * ONEOVERWGS84SEMIMAJOR * RADIANStoDEGREES);

            return result;
        }
    }
}
